"""
    establish_connection(host, user, password, dbname; port=3306) -> MySQL.Connection

Establish a connection to a MySQL/MariaDB database.

# Arguments
- `host::String`: Database host
- `user::String`: Username
- `password::String`: Password
- `dbname::String`: Database name
- `port::Int=3306`: Port number (default is 3306)

# Returns
- `MySQL.Connection`: A connection object to the database.
"""
function establish_connection(host::String, user::String, password::String, dbname::String; port::Int=3306)
    conn = MySQL.Connection(host, user, password, dbname, port, "nothing")
    return conn
end

"""
    query_connection(conn::MySQL.Connection, query::String) -> DataFrame

Executes a SQL query using a MySQL connection and returns the results as a DataFrame.

This function executes the given SQL query on the provided MySQL connection. 
It fetches the query results, converting all data to strings, and returns them in a DataFrame. 
Each column in the DataFrame is of type `String`.

# Arguments
- `conn::MySQL.Connection`: The MySQL connection to use for executing the query.
- `query::String`: The SQL query string to be executed.

# Returns
- `DataFrame`: A DataFrame containing the query results, with all data converted to strings.

# Notes
- The function assumes the connection (`conn`) is already open and valid.
- It converts all columns in the query result to `String` to avoid type interpretation issues.
"""
function query_connection(conn::MySQL.Connection, query::String)
    cursor = DBInterface.execute(conn, query)
    cursor.types = [String for _ in cursor.types]
    rows = Tables.rows(cursor)
    # Initialize an empty DataFrame
    df = DataFrame()
    # Process rows
    for row in rows
        # If df is empty, initialize columns
        if ncol(df) == 0
            for col_name in cursor.names
                df[!, Symbol(col_name)] = String[]
            end
        end
        # Process the row
        row_data = [string(getproperty(row, Symbol(col_name))) for col_name in cursor.names]
        push!(df, row_data)
    end
    return df
end

"""
    close_connection(conn::MySQL.Connection)

Close the given MySQL/MariaDB database connection.

# Arguments
- `conn::MySQL.Connection`: The connection object to close.
"""
function close_connection(conn::MySQL.Connection)
    DBInterface.close!(conn)
end